from sanic.views import HTTPMethodView
from sanic.response import json
from naas.ntypes import t_health
from naas.runner.env_var import n_env
import requests


def get_latest_version():
    try:
        r = requests.get("https://pypi.python.org/pypi/naas/json")
        r.raise_for_status()
        response = r.json()
        version = (
            response["urls"][0]["filename"].replace("naas-", "").replace(".tar.gz", "")
        )
        return version
    except:  # noqa: E722
        return ""


class EnvController(HTTPMethodView):
    def __init__(self, *args, **kwargs):
        super(EnvController, self).__init__(*args, **kwargs)

    async def get(self, request):

        env = {
            "status": t_health,
            "version": n_env.version,
            "latest_version": get_latest_version(),
            "NAAS_BASE_PATH": n_env.path_naas_folder,
            "NOTIFICATIONS_API": n_env.notif_api,
            "JUPYTERHUB_USER": n_env.user,
            "JUPYTER_SERVER_ROOT": n_env.server_root,
            "JUPYTERHUB_URL": n_env.hub_base,
            "PUBLIC_PROXY_API": n_env.proxy_api,
            "TZ": n_env.tz,
        }
        return json(env)



from .ntypes import t_dependency, t_add, t_update, t_delete
from .manager import Manager
import pandas as pd


class Dependency:
    naas = None
    role = t_dependency
    manager = None

    def __init__(self):
        self.manager = Manager(t_dependency)
        self.path = self.manager.path

    def list(self, path=None):
        return self.manager.list_prod("list_history", path)

    def get(self, path=None, histo=None):
        return self.manager.get_file(path, histo=histo)

    def clear(self, path=None, histo=None):
        return self.manager.clear_file(path, None, histo)

    def currents(self, raw=False):
        json_data = self.manager.get_naas()
        json_filtered = []
        for item in json_data:
            if item["type"] == self.role and item["status"] != t_delete:
                if raw:
                    json_filtered.append(item)
                else:
                    json_filtered.append({"path": item["path"]})
        if raw is False:
            df = pd.DataFrame(json_filtered)
            return df
        return json_filtered

    def add(self, path=None, debug=False):
        if self.manager.is_production():
            print("No add done, you are in production\n")
            return self.manager.get_path(path)
        current_file = self.manager.get_path(path)
        status = t_add
        try:
            self.manager.get_value(current_file, False)
            status = t_update
        except:  # noqa: E722
            pass
        self.manager.add_prod(
            {
                "type": self.role,
                "status": status,
                "path": current_file,
                "params": {},
                "value": "Only internal",
            },
            debug,
        )
        print("👌 Well done! Your Dependency has been sent to production. \n")
        print('PS: to remove the "Dependency" feature, just replace .add by .delete')

    def delete(self, path=None, all=True, debug=False):
        if self.manager.is_production():
            print("No delete done, you are in production\n")
            return
        current_file = self.manager.get_path(path)
        self.manager.del_prod({"type": self.role, "path": current_file}, debug)
        print("🗑 Done! Your Dependency has been remove from production.\n")
        if all is True:
            self.clear(current_file, "all")
